from returns.result import Success

from mediathequeroubaix.get_loans.extract_token import extract_token


def test_ok() -> None:
    html = """
        <div class="panel-body">
            <input type="hidden" id="my_issues" value="abc" />
        </div>
    """

    assert extract_token(html) == Success("abc")


def test_empty_html() -> None:
    html = ""

    assert str(extract_token(html)) == "<Failure: Token not found>"


def test_empty_token() -> None:
    html = """
        <div class="panel-body">
            <input type="hidden" id="my_issues" value="" />
        </div>
    """

    assert str(extract_token(html)) == "<Failure: Token not found>"


def test_two_tokens() -> None:
    html = """
        <div class="panel-body">
            <input type="hidden" id="my_issues" value="first" />
            <input type="hidden" id="my_issues" value="another" />
        </div>
    """

    assert extract_token(html) == Success("first")


def test_contain_equal() -> None:
    html = """
        <div class="panel-body">
            <input type="hidden" id="my_issues" value="b64str=" />
        </div>
    """

    assert extract_token(html) == Success("b64str=")



from dataclasses import dataclass
from typing import NewType

from requests import Session

Username = NewType("Username", str)


@dataclass(frozen=True)
class AuthenticatedSession:
    session: Session
    username: Username



from mediathequeroubaix.app import app
from mediathequeroubaix.config import APP_NAME

# Support calling using `python -m`
# https://typer.tiangolo.com/tutorial/package/#support-python-m-optional
app(APP_NAME)



import re

from returns.result import Failure, ResultE, Success

from mediathequeroubaix.auth.authenticated_session import Username


def get_user(html: str) -> ResultE[Username]:
    match = re.search(
        r'<a class="connect" href="/espace_personnel">\s*'
        r'<em class="icon icon-profile-white"></em>\s*'
        r"<div>\s*"
        r"<p>(.+?)</p>\s*"
        r"</div>",
        html,
    )
    if match:
        s = match.group(1)
        return Success(Username(s))
    return Failure(ValueError("User not found"))



import typer
from returns.io import IOFailure, IOResultE, IOSuccess
from returns.pipeline import flow
from returns.pointfree import bind_ioresult
from returns.result import Success
from rich import box, print
from rich.table import Table

from mediathequeroubaix.auth.authenticate import authenticate
from mediathequeroubaix.config import User, get_config
from mediathequeroubaix.get_loans.get_loans import get_loans
from mediathequeroubaix.get_loans.loan import Loans

app = typer.Typer()


@app.command(name="list")
def list_loans() -> None:
    match get_config():
        case IOSuccess(Success(config)):
            if not config.users:
                print("❌ No user defined in configuration")
                raise typer.Exit(1)

            for user in config.users:
                print(f"Getting loans of user '{user.login}'")
                match _get_user_loans(user):
                    case IOSuccess(Success(loans)):
                        _pretty_print(loans)
                    case IOFailure(failure):
                        print(f"❌ Unable to get loans of user '{user.login}'", failure)
        case IOFailure(failure):
            print("❌ Unable to read configuration!", failure)
            raise typer.Exit(1)


def _get_user_loans(user: User) -> IOResultE[Loans]:
    return flow(
        user,
        authenticate,
        bind_ioresult(get_loans),
    )


def _pretty_print(loans: Loans) -> None:
    table = Table(
        title=f"{loans.username}: {len(loans.items)} loans",
        title_style="bold magenta",
        box=box.HEAVY_EDGE,
        expand=True,
    )
    table.add_column("#", justify="center")
    table.add_column("Title", style="blue", ratio=2)
    table.add_column("Due date", justify="center")
    table.add_column("Renewable", justify="center")

    for index, loan in enumerate(loans.items):
        renewable = "✅" if loan.renewable else "❌"
        table.add_row(
            f"{index+1:>2}/{len(loans.items)}",
            loan.title,
            f"{loan.date_due:%d/%m/%Y}",
            renewable,
        )

    print(table)



from pydantic.tools import parse_raw_as
from returns.result import safe

from mediathequeroubaix.get_loans.loan import Loan


@safe
def parse_loans(s: str) -> list[Loan]:
    return parse_raw_as(list[Loan], s)



import base64

import requests
from requests_mock import mock
from returns.pipeline import is_successful
from returns.unsafe import unsafe_perform_io

from mediathequeroubaix.auth.authenticated_session import AuthenticatedSession, Username
from mediathequeroubaix.get_loans.get_loans import get_loans


def test_ok(requests_mock: mock) -> None:
    loans = base64.b64encode(
        '[{"issuedate": "2022-10-08T14:30:14", "barcode": "C2500002968", "itemcallnumber": "E BD/LEG", "date_due": "2022-11-19T23:59:00", "renewable": true, "title": "L\'\u00e9preuve d\'Had\u00e8s", "itemnumber": 360556}]'.encode()
    ).decode("utf-8")
    requests_mock.get(
        "http://www.mediathequederoubaix.fr/espace_personnel",
        text=f"""
        Welcome
        <input type="hidden" id="my_issues" value="{loans}" />
    """,
    )

    result = get_loans(AuthenticatedSession(requests.Session(), Username("John Doe")))

    assert is_successful(result)
    actual = unsafe_perform_io(result.unwrap())
    assert actual.username == "John Doe"
    assert len(actual.items) == 1
    assert actual.items[0].title == "L'épreuve d'Hadès"


def test_no_token(requests_mock: mock) -> None:
    requests_mock.get(
        "http://www.mediathequederoubaix.fr/espace_personnel", text="some html"
    )

    result = get_loans(AuthenticatedSession(requests.Session(), Username("John doe")))

    assert is_successful(result) is False



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['mediathequeroubaix',
 'mediathequeroubaix.auth',
 'mediathequeroubaix.get_loans']

package_data = \
{'': ['*']}

install_requires = \
['pydantic>=1.10.2,<2.0.0',
 'python-dotenv>=0.21.0,<0.22.0',
 'requests>=2.28.1,<3.0.0',
 'returns>=0.19.0,<0.20.0',
 'rich>=12.6.0,<13.0.0',
 'typer[all]>=0.7.0,<0.8.0']

entry_points = \
{'console_scripts': ['mediathequeroubaix = mediathequeroubaix.app:app']}

setup_kwargs = {
    'name': 'mediathequeroubaix',
    'version': '1.1.0',
    'description': 'Client for the library of Roubaix (Médiathèque Roubaix)',
    'long_description': '<p align="center" width="100%">\n  <img src="doc/banner.png" alt="MediathequeRoubaix.py"/>\n</p>\n\n# Python CLI for the library of Roubaix (Médiathèque Roubaix)\n\n[![PyPI](https://img.shields.io/pypi/v/mediathequeroubaix?style=flat-square)](https://pypi.python.org/pypi/mediathequeroubaix/)\n[![PyPI - Python Version](https://img.shields.io/pypi/pyversions/mediathequeroubaix?style=flat-square)](https://pypi.python.org/pypi/mediathequeroubaix/)\n[![PyPI - License](https://img.shields.io/pypi/l/mediathequeroubaix?style=flat-square)](https://pypi.python.org/pypi/mediathequeroubaix/)\n\n---\n\n**Source Code**: [https://github.com/tomsquest/mediathequeroubaix.py](https://github.com/tomsquest/mediathequeroubaix.py)\n\n**PyPI**: [https://pypi.org/project/mediathequeroubaix/](https://pypi.org/project/mediathequeroubaix/)\n\n---\n\n<!-- START doctoc generated TOC please keep comment here to allow auto update -->\n<!-- DON\'T EDIT THIS SECTION, INSTEAD RE-RUN doctoc TO UPDATE -->\n## Table of Contents\n\n- [Features](#features)\n  - [Get the loans](#get-the-loans)\n- [Usage](#usage)\n  - [Install](#install)\n  - [Create an initial, sample configuration](#create-an-initial-sample-configuration)\n  - [Display the current configuration](#display-the-current-configuration)\n  - [List the loans](#list-the-loans)\n- [Why I am doing this](#why-i-am-doing-this)\n- [Changelog](#changelog)\n- [Installation](#installation)\n- [Development](#development)\n  - [Releasing](#releasing)\n- [Credits](#credits)\n\n<!-- END doctoc generated TOC please keep comment here to allow auto update -->\n\n## Features\n\nMédiathèqueRoubaix.py is a client for the **libray of Roubaix**, [mediathequederoubaix.fr](http://www.mediathequederoubaix.fr/).  \n\n<p align="center" width="100%">\n  <img src="doc/mr_homepage.png" alt="Screenshot mediathequederoubaix.fr"/>\n</p>\n\n### Get the loans\n\n1. Display a **list of your loans** and their due date\n2. ...for **many cardholders** at once\n3. and check the **next return date** for all you cards (TODO)\n\n## Usage\n\n### Install\n\n```shell\npip install mediathequederoubaix\n```\n\n### Create an initial, sample configuration\n\n`config create` makes a sample configuration in `$HOME/.config/mediathequederoubaix/config.json` and display the content of the file.  \nThe configuration is initialized with a sample but fake user.\n\n```shell\nmediathequeroubaix config create\n```\n\n<p align="center" width="100%">\n  <img src="doc/cli_config_create.png" alt="Screenshot CLI config create"/>\n</p>\n\n### Display the current configuration\n\n`config show` displays the current configuration.\n\n```shell\nmediathequeroubaix config show\n```\n\n<p align="center" width="100%">\n  <img src="doc/cli_config_show.png" alt="Screenshot CLI config show"/>\n</p>\n\n### List the loans\n\n`loans list` show the list of loans for the users.\n\n```shell\nmediathequeroubaix loans list\n```\n\n<p align="center" width="100%">\n  <img src="doc/cli_loans_list.png" alt="Screenshot CLI loans list"/>\n</p>\n\n## Why I am doing this\n\nI created this project to:\n1. Learn **Functional Programing**\n2. Learn **typed** and **modern** Python\n3. Be able to quickly list and renew my loans (especially when you have many cards)\n\n## Changelog\n\nSee [CHANGELOG.md](CHANGELOG.md)\n\n## Installation\n\n```sh\npip install mediathequeroubaix\n```\n\n## Development\n\n* Clone this repository\n* Requirements:\n  * [Poetry](https://python-poetry.org/)\n  * Python 3.10\n* Create a virtual environment and install the dependencies\n\n```sh\npoetry install\n```\n\n* Activate the virtual environment\n\n```sh\npoetry shell\n```\n\n* Pre-commit\n\n```sh\npre-commit install\n```\n\n* Testing\n\n```sh\npytest\n```\n\n* Check everything\n\n```sh\npre-commit run --all-files\n```\n\n### Releasing\n\nTrigger the [Draft release workflow](https://github.com/tomsquest/mediathequeroubaix.py/actions/workflows/draft_release.yml)\n(press _Run workflow_). This will update the changelog & version and create a GitHub release which is in _Draft_ state.\n\nFind the draft release from the\n[GitHub releases](https://github.com/tomsquest/mediathequeroubaix.py/releases) and publish it. When\n a release is published, it\'ll trigger [release](https://github.com/tomsquest/mediathequeroubaix.py/blob/master/.github/workflows/release.yml) workflow which creates PyPI\n release.\n\n\n## Credits\n\n- Background and color from [PrettySnap](https://prettysnap.app/)\n- Python project bootstrapped using [Wolt template](https://github.com/woltapp/wolt-python-package-cookiecutter)\n- Functional library is [Returns from DRY-Python](https://github.com/dry-python/returns)\n- Tables look great thanks to [Textualize\'s Rich](https://github.com/Textualize/rich)\n- CLI screenshot pimped with [ShowCode.app](https://showcode.app)\n',
    'author': 'Thomas Queste',
    'author_email': 'tom@tomsquest.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://tomsquest.github.io/mediathequeroubaix.py',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.10.0,<3.11.0',
}


setup(**setup_kwargs)
